@@ -33,6 +33,7 @@ from tacker._i18n import _
 from tacker.common import driver_manager
 from tacker.common import log
 from tacker.common import utils
+from tacker import context as t_context
 from tacker.db.nfvo import nfvo_db_plugin
 from tacker.db.nfvo import ns_db
 from tacker.db.nfvo import vnffg_db
@@ -109,14 +110,12 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         try:
             self._vim_drivers.invoke(vim_type,
                                      'register_vim',
-                                     context=context,
                                      vim_obj=vim_obj)
             res = super(NfvoPlugin, self).create_vim(context, vim_obj)
         except Exception:
             with excutils.save_and_reraise_exception():
                 self._vim_drivers.invoke(vim_type,
                                          'delete_vim_auth',
-                                         context=context,
                                          vim_id=vim_obj['id'],
                                          auth=vim_obj['auth_cred'])
 
@@ -170,7 +169,6 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 # Notice: vim_obj may be updated in vim driver's
                 self._vim_drivers.invoke(vim_type,
                                          'register_vim',
-                                         context=context,
                                          vim_obj=vim_obj)
                 new_auth_created = True
 
@@ -185,7 +183,6 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 try:
                     self._vim_drivers.invoke(vim_type,
                                              'delete_vim_auth',
-                                             context=context,
                                              vim_id=old_vim_obj['id'],
                                              auth=old_vim_obj['auth_cred'])
                 except Exception as ex:
@@ -200,7 +197,6 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                     # delete new-created vim auth, old auth is still used.
                     self._vim_drivers.invoke(vim_type,
                                              'delete_vim_auth',
-                                             context=context,
                                              vim_id=vim_obj['id'],
                                              auth=vim_obj['auth_cred'])
 
@@ -209,7 +205,6 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         vim_obj = self._get_vim(context, vim_id)
         self._vim_drivers.invoke(vim_obj['type'],
                                  'deregister_vim',
-                                 context=context,
                                  vim_obj=vim_obj)
         try:
             auth_dict = self.get_auth_dict(context)
@@ -301,6 +296,13 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
 
             self.validate_vnffg_properties(template['template'])
 
+            template_yaml = template['template']['vnffgd']
+            if not template.get('description'):
+                template['description'] = template_yaml.get('description', '')
+            if not template.get('name') and 'metadata' in template_yaml:
+                template['name'] = template_yaml['metadata'].get(
+                    'template_name', '')
+
         return super(NfvoPlugin, self).create_vnffgd(context, vnffgd)
 
     @log.log
@@ -323,7 +325,6 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         nfp = super(NfvoPlugin, self).get_nfp(context,
                                               vnffg_dict['forwarding_paths'])
         sfc = super(NfvoPlugin, self).get_sfc(context, nfp['chain_id'])
-        classifier_dict = dict()
         name_match_list = []
         for classifier_id in nfp['classifier_ids']:
             classifier_dict = super(NfvoPlugin, self).get_classifier(
@@ -345,9 +346,10 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                                              name=item['name'],
                                              fc=item['match'],
                                              auth_attr=vim_obj['auth_cred']))
-            sfc_id = self._vim_drivers.invoke(driver_type,
+            sfc_id, path_id = self._vim_drivers.invoke(driver_type,
                                               'create_chain',
                                               name=vnffg_dict['name'],
+                                              path_id=sfc['path_id'],
                                               vnfs=sfc['chain'],
                                               fc_ids=fc_ids,
                                               symmetrical=sfc['symmetrical'],
@@ -358,7 +360,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         classifiers_map = super(NfvoPlugin, self). \
             create_classifiers_map(nfp['classifier_ids'], fc_ids)
         super(NfvoPlugin, self)._create_vnffg_post(context, sfc_id,
-                                                   classifiers_map,
+                                                   path_id, classifiers_map,
                                                    vnffg_dict)
         super(NfvoPlugin, self)._create_vnffg_status(context, vnffg_dict)
         return vnffg_dict
@@ -512,14 +514,13 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         vim_obj = self.get_vim(context, vim_id['vim_id'], mask_password=False)
         if vim_obj is None:
             raise nfvo.VimFromVnfNotFoundException(vnf_id=vnf_id)
-        self._build_vim_auth(context, vim_obj)
+        self._build_vim_auth(vim_obj)
         return vim_obj
 
-    def _build_vim_auth(self, context, vim_info):
+    def _build_vim_auth(self, vim_info):
         LOG.debug('VIM id is %s', vim_info['id'])
         vim_auth = vim_info['auth_cred']
-        vim_auth['password'] = self._decode_vim_auth(context,
-                                                     vim_info['id'],
+        vim_auth['password'] = self._decode_vim_auth(vim_info['id'],
                                                      vim_auth)
         vim_auth['auth_url'] = vim_info['auth_url']
 
@@ -531,7 +532,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 vim_auth.pop(attr, None)
         return vim_auth
 
-    def _decode_vim_auth(self, context, vim_id, auth):
+    def _decode_vim_auth(self, vim_id, auth):
         """Decode Vim credentials
 
         Decrypt VIM cred, get fernet Key from local_file_system or
@@ -539,10 +540,11 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         """
         cred = auth['password'].encode('utf-8')
         if auth.get('key_type') == 'barbican_key':
+            k_context = t_context.generate_tacker_service_context()
             keystone_conf = CONF.keystone_authtoken
             secret_uuid = auth['secret_uuid']
             keymgr_api = KEYMGR_API(keystone_conf.auth_url)
-            secret_obj = keymgr_api.get(context, secret_uuid)
+            secret_obj = keymgr_api.get(k_context, secret_uuid)
             vim_key = secret_obj.payload
         else:
             vim_key = self._find_vim_key(vim_id)
@@ -657,16 +659,41 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
             if vnfd_name == vnfd['name']:
                 return vnfd['id']
 
+    @log.log
+    def _get_vnffgds_from_nsd(self, nsd_dict):
+        ns_topo = nsd_dict.get('topology_template')
+        vnffgd_templates = dict()
+        if ns_topo and ns_topo.get('groups'):
+            for vnffg_name in ns_topo.get('groups'):
+                vnffgd_template = dict()
+                # TODO(phuoc): add checking in case vnffg_name exists
+                # more than one time.
+                # Constructing vnffgd from nsd, remove imports section
+                vnffgd_template['tosca_definitions_version'] = \
+                    nsd_dict.get('tosca_definitions_version')
+                vnffgd_template['description'] = nsd_dict.get('description')
+                vnffgd_template['topology_template'] = dict()
+                vnffgd_template['topology_template']['groups'] = dict()
+                vnffgd_template['topology_template']['groups'][vnffg_name] = \
+                    ns_topo['groups'].get(vnffg_name)
+                vnffgd_template['topology_template']['node_templates'] = dict()
+                for fp_name in ns_topo['groups'][vnffg_name]['members']:
+                    vnffgd_template['topology_template']['node_templates'][
+                        fp_name] = ns_topo['node_templates'].get(fp_name)
+                vnffgd_templates[vnffg_name] = vnffgd_template
+        return vnffgd_templates
+
     @log.log
     def create_ns(self, context, ns):
-        """Create NS and corresponding VNFs.
+        """Create NS, corresponding VNFs, VNFFGs.
 
         :param ns: ns dict which contains nsd_id and attributes
         This method has 3 steps:
         step-1: substitute all get_input params to its corresponding values
         step-2: Build params dict for substitution mappings case through which
         VNFs will actually substitute their requirements.
-        step-3: Create mistral workflow and execute the workflow
+        step-3: Create mistral workflow to create VNFs, VNFFG and execute the
+        workflow
         """
         ns_info = ns['ns']
         name = ns_info['name']
@@ -693,6 +720,13 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         if not ns['ns']['vim_id']:
             ns['ns']['vim_id'] = vim_res['vim_id']
 
+        # TODO(phuoc): currently, create_ns function does not have
+        # create_ns_pre function, that pre-defines information of a network
+        # service. Creating ns_uuid keeps ns_id for consistency, it should be
+        # provided as return value of create_ns_pre function in ns db.
+        # Generate ns_uuid
+        ns['ns']['ns_id'] = uuidutils.generate_uuid()
+
         # Step-1
         param_values = ns['ns']['attributes'].get('param_values', {})
         if 'get_input' in str(nsd_dict):
@@ -734,6 +768,11 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
             param_values[vnfd_name]['substitution_mappings'][
                 'requirements'] = req_dict
         ns['vnfd_details'] = vnfd_dict
+
+        vnffgd_templates = self._get_vnffgds_from_nsd(nsd_dict)
+        LOG.debug('vnffgd_templates: %s', vnffgd_templates)
+        ns['vnffgd_templates'] = vnffgd_templates
+
         # Step-3
         kwargs = {'ns': ns, 'params': param_values}
 
@@ -741,7 +780,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         workflow = self._vim_drivers.invoke(
             driver_type,
             'prepare_and_create_workflow',
-            resource='vnf',
+            resource='ns',
             action='create',
             auth_dict=self.get_auth_dict(context),
             kwargs=kwargs)
@@ -774,6 +813,8 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 if exec_state == 'SUCCESS' or exec_state == 'ERROR':
                     break
                 mistral_retries = mistral_retries - 1
+            # TODO(phuoc): add more information about error reason in case
+            # of exec_state is 'ERROR'
             error_reason = None
             if mistral_retries == 0 and exec_state == 'RUNNING':
                 error_reason = _(
@@ -795,8 +836,9 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                                      'delete_workflow',
                                      workflow_id=workflow['id'],
                                      auth_dict=self.get_auth_dict(context))
-            super(NfvoPlugin, self).create_ns_post(context, ns_id, exec_obj,
-                                                   vnfd_dict, error_reason)
+            super(NfvoPlugin, self).create_ns_post(
+                context, ns_id, exec_obj, vnfd_dict,
+                vnffgd_templates, error_reason)
 
         self.spawn_n(_create_ns_wait, self, ns_dict['id'],
                      mistral_execution.id)
@@ -831,6 +873,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
     @log.log
     def delete_ns(self, context, ns_id):
         ns = super(NfvoPlugin, self).get_ns(context, ns_id)
+        LOG.debug("Deleting ns: %s", ns)
         vim_res = self.vim_client.get_vim(context, ns['vim_id'])
         super(NfvoPlugin, self).delete_ns_pre(context, ns_id)
         driver_type = vim_res['vim_type']
@@ -840,11 +883,10 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 workflow = self._vim_drivers.invoke(
                     driver_type,
                     'prepare_and_create_workflow',
-                    resource='vnf',
+                    resource='ns',
                     action='delete',
                     auth_dict=self.get_auth_dict(context),
-                    kwargs={
-                        'ns': ns})
+                    kwargs={'ns': ns})
         except nfvo.NoTasksException:
             LOG.warning("No VNF deletion task(s).")
         if workflow:
@@ -878,6 +920,8 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                 if exec_state == 'SUCCESS' or exec_state == 'ERROR':
                     break
                 mistral_retries -= 1
+            # TODO(phuoc): add more information about error reason in case
+            # of exec_state is 'ERROR'
             error_reason = None
             if mistral_retries == 0 and exec_state == 'RUNNING':
                 error_reason = _(
