@@ -15,6 +15,9 @@
 import time
 import yaml
 
+from glanceclient.v2 import client as glance_client
+from keystoneauth1.identity import v3
+from keystoneauth1 import session
 from neutronclient.v2_0 import client as neutron_client
 from novaclient import client as nova_client
 from oslo_config import cfg
@@ -50,40 +53,72 @@ class BaseTackerTest(base.BaseTestCase):
 
     @classmethod
     def get_credentials(cls):
-        vim_params = yaml.load(read_file('local-vim.yaml'))
-        vim_params['auth_url'] += '/v2.0'
+        vim_params = yaml.safe_load(read_file('local-vim.yaml'))
+        vim_params['auth_url'] += '/v3'
         return vim_params
 
     @classmethod
     def tackerclient(cls):
         vim_params = cls.get_credentials()
-        return tacker_client.Client(username=vim_params['username'],
-                                    password=vim_params['password'],
-                                    tenant_name=vim_params['project_name'],
-                                    auth_url=vim_params['auth_url'])
+        auth = v3.Password(auth_url=vim_params['auth_url'],
+            username=vim_params['username'],
+            password=vim_params['password'],
+            project_name=vim_params['project_name'],
+            user_domain_name=vim_params['user_domain_name'],
+            project_domain_name=vim_params['project_domain_name'])
+        verify = 'True' == vim_params.pop('cert_verify', 'False')
+        auth_ses = session.Session(auth=auth, verify=verify)
+        return tacker_client.Client(session=auth_ses)
 
     @classmethod
     def novaclient(cls):
         vim_params = cls.get_credentials()
-        return nova_client.Client('2', vim_params['username'],
-                                  vim_params['password'],
-                                  vim_params['project_name'],
-                                  vim_params['auth_url'])
+        auth = v3.Password(auth_url=vim_params['auth_url'],
+            username=vim_params['username'],
+            password=vim_params['password'],
+            project_name=vim_params['project_name'],
+            user_domain_name=vim_params['user_domain_name'],
+            project_domain_name=vim_params['project_domain_name'])
+        verify = 'True' == vim_params.pop('cert_verify', 'False')
+        auth_ses = session.Session(auth=auth, verify=verify)
+        return nova_client.Client(constants.NOVA_CLIENT_VERSION,
+                                  session=auth_ses)
 
     @classmethod
     def neutronclient(cls):
         vim_params = cls.get_credentials()
-        return neutron_client.Client(**vim_params)
+        auth = v3.Password(auth_url=vim_params['auth_url'],
+            username=vim_params['username'],
+            password=vim_params['password'],
+            project_name=vim_params['project_name'],
+            user_domain_name=vim_params['user_domain_name'],
+            project_domain_name=vim_params['project_domain_name'])
+        verify = 'True' == vim_params.pop('cert_verify', 'False')
+        auth_ses = session.Session(auth=auth, verify=verify)
+        return neutron_client.Client(session=auth_ses)
 
     @classmethod
     def heatclient(cls):
-        data = yaml.load(read_file('local-vim.yaml'))
+        data = yaml.safe_load(read_file('local-vim.yaml'))
         data['auth_url'] = data['auth_url'] + '/v3'
         domain_name = data.pop('domain_name')
         data['user_domain_name'] = domain_name
         data['project_domain_name'] = domain_name
         return clients.OpenstackClients(auth_attr=data).heat
 
+    @classmethod
+    def glanceclient(cls):
+        vim_params = cls.get_credentials()
+        auth = v3.Password(auth_url=vim_params['auth_url'],
+            username=vim_params['username'],
+            password=vim_params['password'],
+            project_name=vim_params['project_name'],
+            user_domain_name=vim_params['user_domain_name'],
+            project_domain_name=vim_params['project_domain_name'])
+        verify = 'True' == vim_params.pop('cert_verify', 'False')
+        auth_ses = session.Session(auth=auth, verify=verify)
+        return glance_client.Client(session=auth_ses)
+
     def wait_until_vnf_status(self, vnf_id, target_status, timeout,
                               sleep_interval):
         start_time = int(time.time())
@@ -171,8 +206,10 @@ class BaseTackerTest(base.BaseTestCase):
                              "List of VNF events are Empty")
         self.assertEqual(cnt, len(vnf_evt_list['vnf_events']))
 
-    def verify_vnfd_events(self, vnfd_id, evt_type, tstamp=None, cnt=1):
+    def verify_vnfd_events(self, vnfd_id, evt_type, res_state,
+                           tstamp=None, cnt=1):
         params = {'resource_id': vnfd_id,
+                  'resource_state': res_state,
                   'resource_type': evt_constants.RES_TYPE_VNFD,
                   'event_type': evt_type}
         if tstamp:
