@@ -37,6 +37,8 @@ vnffg_multi_params = _get_template('vnffg_multi_params.yaml')
 vnffgd_template = yaml.safe_load(_get_template('vnffgd_template.yaml'))
 vnffgd_tosca_template = yaml.safe_load(_get_template(
     'tosca_vnffgd_template.yaml'))
+vnffgd_tosca_no_classifier_template = yaml.safe_load(_get_template(
+    'tosca_vnffgd_no_classifier_template.yaml'))
 vnffgd_tosca_template_for_update = yaml.safe_load(_get_template(
     'tosca_vnffgd_template_for_update.yaml'))
 vnffgd_legacy_template = yaml.safe_load(_get_template(
@@ -189,6 +191,20 @@ def get_dummy_vnffgd_obj_inline():
                         'template_source': 'inline'}}
 
 
+def get_dummy_vnffgd_obj_no_description():
+    return {u'vnffgd': {'name': 'dummy_vnffgd',
+                        'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                        u'template': {u'vnffgd': vnffgd_tosca_template},
+                        'template_source': 'onboarded'}}
+
+
+def get_dummy_vnffgd_obj_no_name():
+    return {u'vnffgd': {'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                        u'template': {u'vnffgd': vnffgd_tosca_template},
+                        'description': 'dummy_vnffgd_description',
+                        'template_source': 'onboarded'}}
+
+
 def get_dummy_vnffg_obj():
     return {'vnffg': {'description': 'dummy_vnffg_description',
                       'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
@@ -199,6 +215,17 @@ def get_dummy_vnffg_obj():
                       'symmetrical': False}}
 
 
+def get_dummy_vnffg_no_classifier_obj():
+    return {'vnffg': {'description': 'dummy_vnffg_no_classifier_description',
+                      'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
+                      'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                      'name': 'dummy_vnffg',
+                      u'attributes': {
+                          u'template': vnffgd_tosca_no_classifier_template},
+                      'vnf_mapping': {},
+                      'symmetrical': False}}
+
+
 def get_dummy_vnffg_obj_inline():
     return {'vnffg': {'description': 'dummy_vnffg_description_inline',
                       'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
@@ -229,10 +256,11 @@ def get_dummy_vnffg_param_obj():
                       'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                       'name': 'dummy_vnffg',
                       u'attributes': {
-                          u'template': vnffgd_tosca_param_template},
+                          u'template': vnffgd_tosca_param_template,
+                          u'param_values':
+                              yaml.safe_load(vnffg_params)
+                      },
                       'vnf_mapping': {},
-                      u'attributes': {u'param_values':
-                          yaml.safe_load(vnffg_params)},
                       'symmetrical': False}}
 
 
@@ -242,10 +270,9 @@ def get_dummy_vnffg_str_param_obj():
                       'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                       'name': 'dummy_vnffg',
                       u'attributes': {
-                          u'template': vnffgd_tosca_param_template},
-                      'vnf_mapping': {},
-                      u'attributes': {
+                          u'template': vnffgd_tosca_param_template,
                           u'param_values': 'value not dict format'},
+                      'vnf_mapping': {},
                       'symmetrical': False}}
 
 
@@ -255,10 +282,11 @@ def get_dummy_vnffg_multi_param_obj():
                       'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                       'name': 'dummy_vnffg',
                       u'attributes': {
-                          u'template': vnffgd_tosca_multi_param_template},
+                          u'template': vnffgd_tosca_multi_param_template,
+                          u'param_values':
+                              yaml.safe_load(vnffg_multi_params)
+                      },
                       'vnf_mapping': {},
-                      u'attributes': {u'param_values':
-                          yaml.safe_load(vnffg_multi_params)},
                       'symmetrical': False}}
 
 
