@@ -54,9 +54,11 @@ def _get_template(name):
 
 class FakeDriverManager(mock.Mock):
     def invoke(self, *args, **kwargs):
-        if any(x in ['create', 'create_chain', 'create_flow_classifier'] for
+        if any(x in ['create', 'create_flow_classifier'] for
                x in args):
             return uuidutils.generate_uuid()
+        elif 'create_chain' in args:
+            return uuidutils.generate_uuid(), uuidutils.generate_uuid()
         elif 'execute_workflow' in args:
             mock_execution = mock.Mock()
             mock_execution.id.return_value = \
@@ -65,21 +67,20 @@ class FakeDriverManager(mock.Mock):
         elif ('prepare_and_create_workflow' in args and
               'delete' == kwargs['action'] and
               DUMMY_NS_2 == kwargs['kwargs']['ns']['id']):
-            raise nfvo.NoTasksException()
+            raise nfvo.NoTasksException(action=kwargs['action'],
+                                        resource=kwargs['kwargs']['ns']['id'])
         elif ('prepare_and_create_workflow' in args and
               'create' == kwargs['action'] and
               utils.DUMMY_NS_2_NAME == kwargs['kwargs']['ns']['ns']['name']):
-            raise nfvo.NoTasksException()
+            raise nfvo.NoTasksException(
+                action=kwargs['action'],
+                resource=kwargs['kwargs']['ns']['ns']['name'])
 
 
 def get_by_name():
     return False
 
 
-def get_by_id():
-    return False
-
-
 def dummy_get_vim_auth(*args, **kwargs):
     return {'vim_auth': {u'username': u'admin', 'password': 'devstack',
                          u'project_name': u'nfv', u'user_id': u'',
@@ -290,24 +291,25 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             res_state=mock.ANY, res_type=constants.RES_TYPE_VIM,
             tstamp=mock.ANY)
         self._driver_manager.invoke.assert_any_call(
-            vim_type, 'register_vim',
-            context=self.context, vim_obj=vim_dict['vim'])
+            vim_type, 'register_vim', vim_obj=vim_dict['vim'])
         self.assertIsNotNone(res)
         self.assertEqual(SECRET_PASSWORD, res['auth_cred']['password'])
         self.assertIn('id', res)
         self.assertIn('placement_attr', res)
         self.assertIn('created_at', res)
         self.assertIn('updated_at', res)
+        self.assertEqual(False, res['is_default'])
+        self.assertEqual('openstack', res['type'])
 
     def test_delete_vim(self):
         self._insert_dummy_vim()
         vim_type = u'openstack'
         vim_id = '6261579e-d6f3-49ad-8bc3-a9cb974778ff'
+        self.context.tenant_id = 'ad7ebc56538745a08ef7c5e97f8bd437'
         vim_obj = self.nfvo_plugin._get_vim(self.context, vim_id)
         self.nfvo_plugin.delete_vim(self.context, vim_id)
         self._driver_manager.invoke.assert_called_once_with(
             vim_type, 'deregister_vim',
-            context=self.context,
             vim_obj=vim_obj)
         self._cos_db_plugin.create_event.assert_called_with(
             self.context, evt_type=constants.RES_EVT_DELETE, res_id=mock.ANY,
@@ -323,6 +325,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         vim_auth_username = vim_dict['vim']['auth_cred']['username']
         vim_project = vim_dict['vim']['vim_project']
         self._insert_dummy_vim()
+        self.context.tenant_id = 'ad7ebc56538745a08ef7c5e97f8bd437'
         res = self.nfvo_plugin.update_vim(self.context, vim_dict['vim']['id'],
                                           vim_dict)
         vim_obj = self.nfvo_plugin._get_vim(
@@ -330,7 +333,6 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         vim_obj['updated_at'] = None
         self._driver_manager.invoke.assert_called_with(
             vim_type, 'register_vim',
-            context=self.context,
             vim_obj=vim_obj)
         self.assertIsNotNone(res)
         self.assertIn('id', res)
@@ -353,6 +355,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         vim_auth_username = vim_dict['vim']['auth_cred']['username']
         vim_project = vim_dict['vim']['vim_project']
         self._insert_dummy_vim_barbican()
+        self.context.tenant_id = 'ad7ebc56538745a08ef7c5e97f8bd437'
         old_vim_obj = self.nfvo_plugin._get_vim(
             self.context, vim_dict['vim']['id'])
         res = self.nfvo_plugin.update_vim(self.context, vim_dict['vim']['id'],
@@ -362,7 +365,6 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         vim_obj['updated_at'] = None
         self._driver_manager.invoke.assert_called_with(
             vim_type, 'delete_vim_auth',
-            context=self.context,
             vim_id=vim_obj['id'],
             auth=old_vim_obj['auth_cred'])
         self.assertIsNotNone(res)
@@ -439,6 +441,18 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         session.flush()
         return vnffg_template
 
+    def _insert_dummy_vnffg_no_classifier_template(self):
+        session = self.context.session
+        vnffg_template = vnffg_db.VnffgTemplate(
+            id='eb094833-995e-49f0-a047-dfb56aaf7c4e',
+            tenant_id='ad7ebc56538745a08ef7c5e97f8bd437',
+            name='fake_template',
+            description='fake_template_description',
+            template={u'vnffgd': utils.vnffgd_tosca_no_classifier_template})
+        session.add(vnffg_template)
+        session.flush()
+        return vnffg_template
+
     def _insert_dummy_vnffg_duplicate_criteria_template(self):
         session = self.context.session
         vnffg_template = vnffg_db.VnffgTemplate(
@@ -547,6 +561,8 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         self.assertIn('id', result)
         self.assertIn('template', result)
         self.assertIn('template_source', result)
+        self.assertEqual('dummy_vnffgd', result['name'])
+        self.assertEqual('dummy_vnffgd_description', result['description'])
         self.assertEqual('onboarded', result['template_source'])
 
     def test_create_vnffgd_inline(self):
@@ -557,6 +573,24 @@ class TestNfvoPlugin(db_base.SqlTestCase):
         self.assertIn('template', result)
         self.assertEqual('inline', result['template_source'])
 
+    def test_create_vnffgd_no_input_description(self):
+        vnffgd_obj = utils.get_dummy_vnffgd_obj_no_description()
+        result = self.nfvo_plugin.create_vnffgd(self.context, vnffgd_obj)
+        self.assertIsNotNone(result)
+        self.assertIn('id', result)
+        self.assertIn('template', result)
+        self.assertIn('template_source', result)
+        self.assertEqual('Example VNFFG template', result['description'])
+
+    def test_create_vnffgd_no_input_name(self):
+        vnffgd_obj = utils.get_dummy_vnffgd_obj_no_name()
+        result = self.nfvo_plugin.create_vnffgd(self.context, vnffgd_obj)
+        self.assertIsNotNone(result)
+        self.assertIn('id', result)
+        self.assertIn('template', result)
+        self.assertIn('template_source', result)
+        self.assertEqual('example_vnffgd', result['name'])
+
     def test_create_vnffg_pre(self):
         VNFFG_ID = 'ffc1a59b-65bb-4874-94d3-84f639e63c74'
         ACLMATCHCRI_ID = 'ffc1a59b-65bb-4874-94d3-84f639e63c75'
@@ -584,6 +618,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
                 'name': vnffg.get('name'),
                 'description': 'fake_template_description',
                 'vnffgd_id': vnffg.get('vnffgd_id'),
+                'ns_id': None,
                 'attributes': template_db.get('template'),
                 'status': constants.PENDING_CREATE,
                 'vnf_mapping': vnf_mapping}
@@ -618,6 +653,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             self.assertEqual('PENDING_CREATE', result['status'])
             self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
                                                            name=mock.ANY,
+                                                           path_id=mock.ANY,
                                                            vnfs=mock.ANY,
                                                            fc_ids=mock.ANY,
                                                            auth_attr=mock.ANY,
@@ -639,6 +675,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             self.assertEqual('PENDING_CREATE', result['status'])
             self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
                                                            name=mock.ANY,
+                                                           path_id=mock.ANY,
                                                            vnfs=mock.ANY,
                                                            fc_ids=mock.ANY,
                                                            auth_attr=mock.ANY,
@@ -665,6 +702,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             mock_create_vnffgd.assert_called_once_with(mock.ANY, mock.ANY)
             self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
                                                            name=mock.ANY,
+                                                           path_id=mock.ANY,
                                                            vnfs=mock.ANY,
                                                            fc_ids=mock.ANY,
                                                            auth_attr=mock.ANY,
@@ -686,18 +724,40 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             self.assertEqual('PENDING_CREATE', result['status'])
             self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
                                                            name=mock.ANY,
+                                                           path_id=mock.ANY,
                                                            vnfs=mock.ANY,
                                                            fc_ids=mock.ANY,
                                                            auth_attr=mock.ANY,
                                                            symmetrical=mock.ANY
                                                            )
 
-    @mock.patch.object(nfvo_plugin.NfvoPlugin, '_get_by_id')
-    def test_create_vnffg_param_value_format_error(self, mock_get_by_id):
+    def test_create_vnffg_no_classifier(self):
         with patch.object(TackerManager, 'get_service_plugins') as \
                 mock_plugins:
             mock_plugins.return_value = {'VNFM': FakeVNFMPlugin()}
-            mock_get_by_id.value = get_by_id()
+            mock.patch('tacker.common.driver_manager.DriverManager',
+                       side_effect=FakeDriverManager()).start()
+            self._insert_dummy_vnffg_no_classifier_template()
+            vnffg_obj = utils.get_dummy_vnffg_no_classifier_obj()
+            result = self.nfvo_plugin.create_vnffg(self.context, vnffg_obj)
+            self.assertIsNotNone(result)
+            self.assertIn('id', result)
+            self.assertIn('status', result)
+            self.assertEqual('PENDING_CREATE', result['status'])
+            self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
+                                                           name=mock.ANY,
+                                                           path_id=mock.ANY,
+                                                           vnfs=mock.ANY,
+                                                           fc_ids=mock.ANY,
+                                                           auth_attr=mock.ANY,
+                                                           symmetrical=mock.ANY
+                                                           )
+
+    def test_create_vnffg_param_value_format_error(self):
+        with patch.object(TackerManager, 'get_service_plugins') as \
+                mock_plugins:
+            mock_plugins.return_value = {'VNFM': FakeVNFMPlugin()}
+            self._insert_dummy_vnffg_param_template()
             vnffg_obj = utils.get_dummy_vnffg_str_param_obj()
             self.assertRaises(nfvo.VnffgParamValueFormatError,
                               self.nfvo_plugin.create_vnffg,
@@ -713,16 +773,6 @@ class TestNfvoPlugin(db_base.SqlTestCase):
                               self.nfvo_plugin.create_vnffg,
                               self.context, vnffg_obj)
 
-    def test_create_vnffg_param_value_not_use(self):
-        with patch.object(TackerManager, 'get_service_plugins') as \
-                mock_plugins:
-            mock_plugins.return_value = {'VNFM': FakeVNFMPlugin()}
-            self._insert_dummy_vnffg_param_template()
-            vnffg_obj = utils.get_dummy_vnffg_multi_param_obj()
-            self.assertRaises(nfvo.VnffgParamValueNotUsed,
-                              self.nfvo_plugin.create_vnffg,
-                              self.context, vnffg_obj)
-
     def test_create_vnffg_vnf_mapping(self):
         with patch.object(TackerManager, 'get_service_plugins') as \
                 mock_plugins:
@@ -738,6 +788,7 @@ class TestNfvoPlugin(db_base.SqlTestCase):
             self.assertEqual('PENDING_CREATE', result['status'])
             self._driver_manager.invoke.assert_called_with(mock.ANY, mock.ANY,
                                                            name=mock.ANY,
+                                                           path_id=mock.ANY,
                                                            vnfs=mock.ANY,
                                                            fc_ids=mock.ANY,
                                                            auth_attr=mock.ANY,
