@@ -28,8 +28,8 @@ from tacker.vnfm.mgmt_drivers import constants as mgmt_constants
 
 LOG = logging.getLogger(__name__)
 OPTS = [
-    cfg.StrOpt('user', default='root', help=_('user name to login openwrt')),
-    cfg.StrOpt('password', default='', help=_('password to login openwrt')),
+    cfg.StrOpt('user', default='root', help=_('User name to login openwrt')),
+    cfg.StrOpt('password', default='', help=_('Password to login openwrt')),
 ]
 cfg.CONF.register_opts(OPTS, 'openwrt')
 
@@ -56,8 +56,11 @@ class DeviceMgmtOpenWRT(abstract_driver.DeviceMGMTAbstractDriver):
     def _config_service(self, mgmt_ip_address, service, config):
         user = cfg.CONF.openwrt.user
         password = cfg.CONF.openwrt.password
+        package = service
+        if service == "dhcp":
+            package = "dnsmasq"
         try:
-            cmd = "uci import %s; /etc/init.d/%s restart" % (service, service)
+            cmd = "uci import %s; /etc/init.d/%s restart" % (service, package)
             LOG.debug('execute command: %(cmd)s on mgmt_ip_address '
                       '%(mgmt_ip)s',
                       {'cmd': cmd,
@@ -91,7 +94,7 @@ class DeviceMgmtOpenWRT(abstract_driver.DeviceMGMTAbstractDriver):
         for vdu, vdu_dict in vdus_config_dict.items():
             config = vdu_dict.get('config', {})
             for key, conf_value in config.items():
-                KNOWN_SERVICES = ('firewall', 'network')
+                KNOWN_SERVICES = ('firewall', 'network', 'dhcp', 'qos')
                 if key not in KNOWN_SERVICES:
                     continue
                 mgmt_ip_address = mgmt_url.get(vdu, '')
