@@ -32,11 +32,11 @@ from tacker.common import utils
 from tacker.db.vnfm import vnfm_db
 from tacker.extensions import vnfm
 from tacker.plugins.common import constants
+from tacker.tosca import utils as toscautils
 from tacker.vnfm.mgmt_drivers import constants as mgmt_constants
 from tacker.vnfm import monitor
 from tacker.vnfm import vim_client
 
-from tacker.tosca import utils as toscautils
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -115,7 +115,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
     """
     OPTS_INFRA_DRIVER = [
         cfg.ListOpt(
-            'infra_driver', default=['noop', 'openstack'],
+            'infra_driver', default=['noop', 'openstack', 'kubernetes'],
             help=_('Hosting vnf drivers tacker plugin will use')),
     ]
     cfg.CONF.register_opts(OPTS_INFRA_DRIVER, 'tacker')
@@ -143,6 +143,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             cfg.CONF.tacker.policy_action)
         self._vnf_monitor = monitor.VNFMonitor(self.boot_wait)
         self._vnf_alarm_monitor = monitor.VNFAlarmMonitor()
+        self._vnf_app_monitor = monitor.VNFAppMonitor()
 
     def spawn_n(self, function, *args, **kwargs):
         self._pool.spawn_n(function, *args, **kwargs)
@@ -215,6 +216,12 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
 
         vnfd_dict['mgmt_driver'] = toscautils.get_mgmt_driver(
             tosca)
+
+        if vnfd_dict['mgmt_driver'] not in cfg.CONF.tacker.mgmt_driver:
+            LOG.error("Invalid mgmt_driver in TOSCA template")
+            raise vnfm.InvalidMgmtDriver(
+                mgmt_driver_name=vnfd_dict['mgmt_driver'])
+
         LOG.debug('vnfd %s', vnfd)
 
     def add_vnf_to_monitor(self, context, vnf_dict):
@@ -247,6 +254,10 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                     vnf_dict['attributes'].update(alarm_url)
                     break
 
+    def add_vnf_to_appmonitor(self, context, vnf_dict):
+        appmonitor = self._vnf_app_monitor.create_app_dict(context, vnf_dict)
+        self._vnf_app_monitor.add_to_appmonitor(appmonitor, vnf_dict)
+
     def config_vnf(self, context, vnf_dict):
         config = vnf_dict['attributes'].get('config')
         if not config:
@@ -327,8 +338,10 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             context, vnf) if not vnf.get('id') else vnf
         vnf_id = vnf_dict['id']
         LOG.debug('vnf_dict %s', vnf_dict)
-        self.mgmt_create_pre(context, vnf_dict)
-        self.add_alarm_url_to_vnf(context, vnf_dict)
+        if driver_name == 'openstack':
+            self.mgmt_create_pre(context, vnf_dict)
+            self.add_alarm_url_to_vnf(context, vnf_dict)
+
         try:
             instance_id = self._vnf_manager.invoke(
                 driver_name, 'create', plugin=self,
@@ -361,6 +374,14 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                              'service_types': [{'service_type': 'vnfd'}]}}
             vnf_info['vnfd_id'] = self.create_vnfd(context, vnfd).get('id')
 
+        infra_driver, vim_auth = self._get_infra_driver(context, vnf_info)
+        if infra_driver not in self._vnf_manager:
+            LOG.debug('unknown vim driver '
+                      '%(infra_driver)s in %(drivers)s',
+                      {'infra_driver': infra_driver,
+                       'drivers': cfg.CONF.tacker.infra_driver})
+            raise vnfm.InvalidInfraDriver(vim_name=infra_driver)
+
         vnf_attributes = vnf_info['attributes']
         if vnf_attributes.get('param_values'):
             param = vnf_attributes['param_values']
@@ -380,18 +401,15 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 vnf_attributes['config'] = yaml.safe_dump(config)
             else:
                 self._report_deprecated_yaml_str()
-        infra_driver, vim_auth = self._get_infra_driver(context, vnf_info)
-        if infra_driver not in self._vnf_manager:
-            LOG.debug('unknown vim driver '
-                      '%(infra_driver)s in %(drivers)s',
-                      {'infra_driver': infra_driver,
-                       'drivers': cfg.CONF.tacker.infra_driver})
-            raise vnfm.InvalidInfraDriver(vim_name=infra_driver)
 
         vnf_dict = self._create_vnf(context, vnf_info, vim_auth, infra_driver)
 
         def create_vnf_wait():
             self._create_vnf_wait(context, vnf_dict, vim_auth, infra_driver)
+
+            if 'app_monitoring_policy' in vnf_dict['attributes']:
+                self.add_vnf_to_appmonitor(context, vnf_dict)
+
             if vnf_dict['status'] is not constants.ERROR:
                 self.add_vnf_to_monitor(context, vnf_dict)
             self.config_vnf(context, vnf_dict)
@@ -794,14 +812,8 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         return trigger
 
     def _handle_vnf_monitoring(self, context, trigger):
-        vnf_dict = trigger['vnf']
-        if trigger['action_name'] in constants.DEFAULT_ALARM_ACTIONS:
-            action = trigger['action_name']
-            LOG.debug('vnf for monitoring: %s', vnf_dict)
-            self._vnf_action.invoke(
-                action, 'execute_action', plugin=self, context=context,
-                vnf_dict=vnf_dict, args={})
 
+        vnf_dict = trigger['vnf']
         # Multiple actions support
         if trigger.get('policy_actions'):
             policy_actions = trigger['policy_actions']
@@ -818,9 +830,9 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                     bckend_policy_type = bckend_policy['type']
                     if bckend_policy_type == constants.POLICY_SCALING:
                         if vnf_dict['status'] != constants.ACTIVE:
-                            LOG.info(_("Scaling Policy action"
-                                       "skipped due to status:"
-                                       "%(status)s for vnf: %(vnfid)s"),
+                            LOG.info(_("Scaling Policy action "
+                                       "skipped due to status "
+                                       "%(status)s for vnf %(vnfid)s"),
                                      {"status": vnf_dict['status'],
                                       "vnfid": vnf_dict['id']})
                             return
