@@ -151,18 +151,18 @@ class Server(object):
 
         if CONF.use_ssl:
             if not os.path.exists(CONF.ssl_cert_file):
-                raise RuntimeError(_("Unable to find ssl_cert_file "
+                raise RuntimeError(_("Unable to find ssl_cert_file"
                                      ": %s") % CONF.ssl_cert_file)
 
             # ssl_key_file is optional because the key may be embedded in the
             # certificate file
             if CONF.ssl_key_file and not os.path.exists(CONF.ssl_key_file):
                 raise RuntimeError(_("Unable to find "
-                                     "ssl_key_file : %s") % CONF.ssl_key_file)
+                                     "ssl_key_file: %s") % CONF.ssl_key_file)
 
             # ssl_ca_file is optional
             if CONF.ssl_ca_file and not os.path.exists(CONF.ssl_ca_file):
-                raise RuntimeError(_("Unable to find ssl_ca_file "
+                raise RuntimeError(_("Unable to find ssl_ca_file"
                                      ": %s") % CONF.ssl_ca_file)
 
         def wrap_ssl(sock):
@@ -228,8 +228,8 @@ class Server(object):
         else:
             # Minimize the cost of checking for child exit by extending the
             # wait interval past the default of 0.01s.
-            self._launcher = common_service.ProcessLauncher(CONF,
-                                                            wait_interval=1.0)
+            self._launcher = common_service.ProcessLauncher(
+                CONF, wait_interval=1.0, restart_method='mutate')
             self._server = WorkerService(self, application)
             self._launcher.launch_service(self._server, workers=workers)
 
